
Hello there;)


- This python program has only one function called "creat_100_times".
- Which use the os and pathlib library to crate over 100 folder each one iside other.
- And then take a custom file and move it into the last folder using the shutil module.
- Mybe you gonna use it to hide a game from your siblings or just make a file more secured or ...
- To use this module you nedd to have the library "send2trash" installed or it's gonna be an error.