Documents
=========

- [Checks](checks.md): Full list of all checks done by actionlint with example inputs, outputs, and playground links.
- [Installation](install.md): Installation instructions. Prebuilt binaries, Homebrew package, a Docker image, building from
  source, a download script (for CI) are available.
- [Usage](usage.md): How to use `actionlint` command locally or on GitHub Actions, the online playground, an official Docker
  image, and integrations with reviewdog, Problem Matchers, super-linter, pre-commit.
- [Configuration](config.md): How to configure actionlint behavior. Currently only labels of self-hosted runners can be
  configured.
- [Go API](api.md): How to use actionlint as Go library.
- [References](reference.md): Links to resources.
