# Contributing to gotest.tools

Thank you for your interest in contributing to the project! Below are some
suggestions which may make the process easier.

## Pull requests

Pull requests for new features should generally be preceded by an issue
explaining the feature and why it is necessary.

Pull requests for bug fixes are always appreciated. They should almost always
include a test which fails without the bug fix.
