# container port

<!---MARKER_GEN_START-->
List port mappings or a specific mapping for the container

### Aliases

`docker container port`, `docker port`


<!---MARKER_GEN_END-->

## Description

See [docker port](port.md) for more information.
