# container pause

<!---MARKER_GEN_START-->
Pause all processes within one or more containers

### Aliases

`docker container pause`, `docker pause`


<!---MARKER_GEN_END-->

## Description

See [docker pause](pause.md) for more information.
