---
title: Test
---

## About events that trigger workflows

## `check_run`

oops first cell does not contain [`check_run`](/webhooks/event-payloads/#check_run)!

| Webhook event payload | Activity types | `GITHUB_SHA` | `GITHUB_REF` |
| --------------------- | -------------- | ------------ | -------------|
| wow | - `created`<br/>- `rerequested`<br/>- `completed` | Last commit on default branch | Default branch |
