title
-----

### Context availability

| Workflow key | Context | Special functions |
| ---- | ------- | ----------------- |
| <code>concurrency</code> | <code>{% ifhoge ... %}github, inputs{% else %}foo, bar{% endif %}</code> | |
